from .teiauxiliary import read_yaml, load_directories, create_file_list
import pkg_resources
import os


filenames = create_file_list()

print(filenames)


TT_CFG = read_yaml(filenames['config']['tei_transformer.yaml'])
HTMLT_CFG = read_yaml(filenames['config']['html_transformer.yaml'])
DOCX_CFG = read_yaml(filenames['config']["docx_constructor.yaml"])

SCENARIOS = ['drama', 'plain']

doc_temp = filenames['templates']['document7.html']
file_desc = filenames['xsl']['fileDesc_all.xsl']



# SCHEMA_PATH = TT_CFG["PATHS"]["schema_dir"]
SCENARIOS = ['drama', 'plain']



# -*- coding: utf-8 -*-

import os
import copy
import logging

from .setup import filenames, TT_CFG
from lxml import etree
from .pathvalidator import PathValidator

SCHEMA_ROOT = etree.XML(TT_CFG['SCHEMA']['root'])


class SchemaValidator:

    @staticmethod
    def load_rng_schema(schema_path):
        """
        Load rng schema.
        :param schema_path: str
        :return: lxml object
        """
        relaxng_doc = etree.parse(schema_path)
        relaxng = etree.RelaxNG(relaxng_doc)
        return relaxng

    @staticmethod
    def load_dtd_schema(schema_path):
        """
        Load dtd schema.
        :param schema_path: str
        :return: lxml object
        """
        dtd = etree.DTD(file=schema_path)
        return dtd

    @staticmethod
    def load_xsd_schema(schema_path):
        """
        Load xsd schema.
        :param schema_path: str
        :return: lxml object
        """
        xmlschema_doc = etree.parse(schema_path)
        xmlschema = etree.XMLSchema(xmlschema_doc)
        return xmlschema

    def load_schema(self, schema_path, schema_type):
        """
        Load schema.
        :param schema_path: str
        :param schema_type: str
        :return: lxml object
        """
        parser_func = "load_{}_schema".format(schema_type)
        schema_parser = getattr(self, parser_func)(schema_path)
        return schema_parser

    def validator_generator(self, scenario):
        """
        Load default schema.
        :param scenario: str
        :return: lxml object
        """
        
        schema_folder = filenames['schema'][scenario][TT_CFG['FORMATS']['default_schema']]
        os.chdir(schema_folder)
        filename = "tei_{}.{}".format(
            scenario, TT_CFG['FORMATS']['default_schema'])
        schema_parser = self.load_schema(
            filename, TT_CFG['FORMATS']['default_schema'])
        return schema_parser

    @staticmethod
    def __get_schema_locations(tei, nsp):
        """
        Get schema location attribute
        :param tei: TEI XML object
        :param nsp: str
        :return: set
        """
        pattern = TT_CFG['SCHEMA']['loc_pattern'].format(nsp)
        schema_locs = set(tei.root.xpath(pattern, namespaces=tei.nsmap))
        return schema_locs

    @staticmethod
    def __add_schema_import(location, schema_root):
        """
        Add schema import line to the schema template.
        :param location: str
        :param schema_root: lxml element
        :return:
        """
        schema_params = location.strip().split()
        xs_import = etree.Element(TT_CFG['SCHEMA']['xs_import'])
        if len(schema_params) != 2:
            raise ValueError("The namespace is required for schemaLocation")
        xs_import.attrib['namespace'] = schema_params[0]
        xs_import.attrib['schemaLocation'] = schema_params[1]
        schema_root.append(xs_import)
        return schema_root

    def __import_schema(self, schema_locs):
        """
        Impotrs schema from location.
        :param schema_locs: list
        :return: lxml element
        """
        schema_root = copy.deepcopy(SCHEMA_ROOT)
        for location in schema_locs:
            schema_root = self.__add_schema_import(location, schema_root)
        return schema_root

    def __check_root_by_namespace(self, tei, nsp):
        """
        Checks root for schema location according to a namespace.
        :param tei: TEI XML object
        :param nsp: str
        :return: lxml object/bool
        """
        schema_locs = self.__get_schema_locations(tei, nsp)
        if schema_locs:
            schema_root = self.__import_schema(schema_locs)
            schema = etree.XMLSchema(schema_root)
            return schema
        return False

    def check_root(self, tei):
        """
        Checks root for schema location according.
        :param tei: TEI XML object
        :return: lxml object/bool
        """
        for nsp in tei.nsmap:
            schema = self.__check_root_by_namespace(tei, nsp)
            if schema: return schema
        return False


class XMLValidator(PathValidator, SchemaValidator):

    @staticmethod
    def __validate_schema(tei, validator):
        """
        Validate xml against schema.
        :param tei: lxml object
        :param validator: lxml object
        :return: bool
        """
        if validator.validate(tei.dom):
            return True
        logging.warning("Invalid TEI")
        return False

    def __validate_against_schema(self, tei, schema_path):
        """
        Validate xml against user schema.
        :param tei: lxml object
        :param schema_path: str
        :return: None
        """
        _, extension = os.path.splitext(schema_path)
        validator = self.load_schema(schema_path, extension[1:])
        self.__validate_schema(tei, validator)

    def __validate_default_schema(self, tei, scenario):
        """
        Validate xml against default schema.
        :param tei: lxml object
        :param scenario: str
        :return: None
        """
        cur_directory = os.getcwd()
        validator = self.validator_generator(scenario)
        self.__validate_schema(tei, validator)
        os.chdir(cur_directory)

    def validate(self, tei, schema_path=None, scenario=None):
        """
        Validate xml
        :param tei: TEI XML object
        :param schema_path: str
        :param scenario: str
        :return: None
        """
        root_validator = self.check_root(tei)
        if schema_path:
            self.validate_path(schema_path, extension_type='schema')
            self.__validate_against_schema(tei, schema_path)
        elif root_validator:
            self.__validate_schema(tei, root_validator)
        elif scenario:
            self.__validate_default_schema(tei, scenario)



# -*- coding: utf-8 -*-
import os
import re

import yaml
import json
import logging

import pkg_resources
from collections import defaultdict


size_pattern = re.compile("(?P<value>(?:\d|\.)+)(?P<measure>[a-z]+)")
rgb_pattern = re.compile("rgb\((?P<r>.+?), (?P<g>.+?), (?P<b>.+?)\)")
CSS_VALUE_PATTERNS = [size_pattern, rgb_pattern]

PROJECT_DIR = os.path.dirname(__file__)
DATA_DIR = os.path.join(PROJECT_DIR, 'data')
RESOURCE = 'testmodule3091'


# def get_files(path):
# 	for file in os.listdir(path):
# 		if not file.startswith('.'):
# 			yield file
# 			
# 
# def create_file_list(data_dir, filenames={}):
# # 	filenames = {}
# 	for dir_ in get_files(DATA_DIR):
# 		dir_path = os.path.join(DATA_DIR, dir_)
# 		filenames[dir_] = defaultdict(list)
# 		for file in get_files(dir_path):
# 			file_path = os.path.join(dir_path, file)
# 			filenames[dir_][file] = file_path
# 	return filenames


def create_file_list(data_dir0, dirname0, filenames={}):
    for file in get_files(data_dir0):
        file_path = os.path.join(data_dir0, file)
        filenames[file] = file_path
    
    for dirname1 in get_dirs(data_dir0):
        dir_path2 = os.path.join(data_dir0, dirname1)
        filenames[dirname1] = create_file_list(dir_path2, dirname1, filenames={})
            
    return filenames

def get_files(path):
    for file in os.listdir(path):
        if not file.startswith('.') and '.' in file:
            yield file
            
def get_dirs(path):
    for file in os.listdir(path):
        if not file.startswith('.') and '.' not in file:
            yield file
    
    


def read_yaml(path):
    """
    Reads yaml file.
    :param path: str
    :return: yaml object
    """
    with open(path) as f:
        data = yaml.safe_load(f)
    return data


def read_file(path):
    """
    Reads file.
    :param path: str
    :return: str
    """
    with open(path, "r", encoding='utf-8') as f:
        data = f.read()
    return data


def write_html(path, html, full_page=False):
    """
    Writes html to file.
    :param path: str
    :param html: lxml object
    :param full_page: bool
    :return: None
    """
    if full_page:
        html.write(path, pretty_print=True)
    else:
        with open(path, 'w', encoding="utf-8") as f:
            f.write(str(html))


def write_json(path, data):
    """
    Write json to file.
    :param path: str
    :param data: dict
    :return: None
    """
    with open(path, "w") as outfile:
        json.dump(data, outfile)


def load_directories(path):
    """
    Get directory names.
    :param path: str
    :return: list
    """
    dirs = [name
        for name in os.listdir(path)
        if os.path.isdir(os.path.join(path, name))
    ]
    return dirs


def get_filename(path):
    """
    Get filename from path.
    :param path: str
    :return: str
    """
    filename = os.path.split(path)[-1]
    filename = os.path.splitext(filename)[0]
    return filename


def prepare_func_name(*args, ftype='parse'):
    """
    Prepares function name.
    :param args: []
    :param ftype: str
    :return: name
    """
    name = "_".join((ftype,) + args)
    name = re.sub("-", "_", name)
    return name


def clean_text(text):
    """
    Preprocess text name.
    :param text: str
    :return: str
    """
    return ' '.join(text.split())


def css_value_parser(line):
    """
    Parse css rule.
    :param line: str
    :return: dict
    """
    global CSS_VALUE_PATTERNS
    for pattern in CSS_VALUE_PATTERNS:
        res = pattern.search(line)
        if res: return res.groupdict()
    return {}


def check_if_rgb(value):
    """
    Checks if color value is RGB.
    :param value: str
    :return: bool
    """
    try:
        cond1 = isinstance(value, str)
        cond2 = len(value) == 3
        cond3 = all(isinstance(x, int) for x in value)
        if cond1 and cond2 and cond3: return True
        return False
    except:
        return False


def check_bool(value, name):
    """
    Checks if variable is bool
    :param value: any
    :param name: str
    :return: None
    """
    if not isinstance(value, bool):
        raise ValueError("Wrong datatype. {} should be bool".format(name))
    return None


def check_str(value, name):
    """
    Checks if variable is str
    :param value: any
    :param name: str
    :return: None
    """
    if not isinstance(value, str):
        raise ValueError("Wrong datatype. {} should be string".format(name))
    return None


def check_iterable(value, name):
    """
    Checks if variable is int
    :param value: any
    :param name: str
    :return: None
    """
    if not isinstance(value, (list, tuple, set)):
        raise ValueError(
            "Wrong datatype. {} should be list, tuple or set".format(name)
        )
    return None


def check_scenario(scenario):
    """
    Checks is scenario is valid.
    :param scenario: str
    :return: str
    """
    if not isinstance(scenario, str):
        raise ValueError("Scenario. Wrong datatype.")
    scenarios = ['drama', 'plain']
    if scenario in scenarios:
        return scenario
    logging.warning("Wrong scenario. Available options: {}".format(*scenarios))
    return 'plain'


def check_extension():
    pass



# -*- coding: utf-8 -*-
import os
from .setup import TT_CFG


class PathValidator:

    """
    Interface for path validation
    """

    @staticmethod
    def if_exists(filepath):
        """
        Checks if file exists.
        :param filepath: str
        :return: bool
        """
        if os.path.isfile(filepath):
            return True
        return False

    @staticmethod
    def check_extension(filepath, extension_type):
        """
        Checks if extension is valid
        :param filepath: str
        :param extension_type: str
        :return: bool
        """
        filename, extension = os.path.splitext(filepath)
        return extension in TT_CFG['FORMATS'][extension_type]

    def validate_path(self, filepath, extension_type):
        """
        Validates path.
        :param filepath: str
        :param extension_type: str
        :return: bool
        """
        exists = self.if_exists(filepath)
        cor_extension = self.check_extension(filepath, extension_type)
        if exists and cor_extension:
            return True
        raise OSError("Incorrect extension or file not found")



# -*- coding: utf-8 -*-
from .xmlvalidators import XMLValidator
from .teiauxiliary import get_filename, check_scenario, check_bool, check_str
from .director import Director
from .teixml import TEIXML
import logging


class TEITransformer:

    """
    Client interface of the algorithm.
    Takes data from the user.
    Transforms according to the input parameters.
    """

    def __init__(self, scenario='plain'):
        """

        :param scenario: str
        """
        self.scenario = check_scenario(scenario)
        self.director = Director(scenario=self.scenario)
        self.validator = XMLValidator()
        self.valid = False

    def load_tei(self, tei_path, schema_path=None):
        """
        Creates an TEIXML object from a file and validates it against schema.
        :param tei_path: str
        :param schema_path: str
        :return: None
        """
        self.valid = False
        self.filename = get_filename(tei_path)
        self.validator.validate_path(tei_path, "file")
        self.tei = TEIXML(tei_path, tei_ns=True)
        self.valid = self.validator.validate(
            self.tei, schema_path=schema_path,
            scenario=self.scenario)

    def transform(self, output_format='html',
                  output_filename=None,
                  enable_valid=True, **kwargs):
        """
        Defines the transformation method.
        Performs a transformation.
        :param output_format: str
        :param output_filename: str
        :param enable_valid: bool
        :param kwargs: arguments for selected transformation method
        :return: None
        """
        check_bool(enable_valid, 'enable_valid')
        if not output_filename:
            output_filename = self.filename
        else:
            check_str(output_filename, 'output_filename')
        if self.valid or enable_valid is False:
            construction_method = self.director.select_construction_method(
                output_format
            )
            getattr(self.director, construction_method)(
                self.tei, output_filename=output_filename, **kwargs)
            logging.warning("The file was successfully transformed.")
            return None
        raise ValueError("Unable to perform conversion for invalid file.")



from setuptools import setup, find_packages

VERSION = '0.2.5' 
DESCRIPTION = 'My first Python package'
LONG_DESCRIPTION = 'My first Python package with a slightly longer description'

# Setting up
setup(
       # the name must match the folder name 'verysimplemodule'
        name="testmodule3091", 
        version=VERSION,
        author="AKS",
        author_email="",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=[
         'beautifulsoup4==4.9.3', 
		 'certifi==2020.12.5', 
		 'chardet==4.0.0', 
		 'cssutils==2.3.0', 
		 'idna==2.10', 
		 'importlib-metadata==4.0.1', 
		 'lxml==4.6.3', 
		 'mypy-extensions==0.4.3', 
		 'python-docx==0.8.11', 
		 'PyYAML==5.4.1', 
		 'qwikidata==0.4.0', 
		 'requests==2.25.1', 
		 'soupsieve==2.2.1', 
		 'typing-extensions==3.10.0.0', 
		 'urllib3==1.26.4', 
		 'zipp==3.4.1'
        ], 
        
        include_package_data=True,
        
        keywords=['python', 'first package'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Education",
            "Programming Language :: Python :: 2",
            "Programming Language :: Python :: 3",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: Microsoft :: Windows",
        ]
)